@@ -3,26 +3,30 @@
 #include <stdlib.h>
 #include <limits.h>
 #include <math.h>
+// needed for FILE:
 #include <stdio.h>
+// For use with R package 'Matrix'
+# include <R_ext/Print.h>
+# define printf Rprintf
 #ifdef MATLAB_MEX_FILE
 #include "mex.h"
 #endif
-#define CS_VER 2		    /* CSparse Version 2.2.2 */
+#define CS_VER 2                    /* CSparse Version 2.2.3 */
 #define CS_SUBVER 2
-#define CS_SUBSUB 2
-#define CS_DATE "Sept 23, 2008"	    /* CSparse release date */
-#define CS_COPYRIGHT "Copyright (c) Timothy A. Davis, 2006-2008"
+#define CS_SUBSUB 3
+#define CS_DATE "Jan 20, 2009"     /* CSparse release date */
+#define CS_COPYRIGHT "Copyright (c) Timothy A. Davis, 2006-2009"
 
 /* --- primary CSparse routines and data structures ------------------------- */
 typedef struct cs_sparse    /* matrix in compressed-column or triplet form */
 {
-    int nzmax ;	    /* maximum number of entries */
-    int m ;	    /* number of rows */
-    int n ;	    /* number of columns */
-    int *p ;	    /* column pointers (size n+1) or col indices (size nzmax) */
-    int *i ;	    /* row indices, size nzmax */
-    double *x ;	    /* numerical values, size nzmax */
-    int nz ;	    /* # of entries in triplet matrix, -1 for compressed-col */
+    int nzmax ;     /* maximum number of entries */
+    int m ;         /* number of rows */
+    int n ;         /* number of columns */
+    int *p ;        /* column pointers (size n+1) or col indices (size nzmax) */
+    int *i ;        /* row indices, size nzmax */
+    double *x ;     /* numerical values, size nzmax */
+    int nz ;        /* # of entries in triplet matrix, -1 for compressed-col */
 } cs ;
 
 cs *cs_add (const cs *A, const cs *B, double alpha, double beta) ;
@@ -50,31 +54,31 @@ void *cs_malloc (int n, size_t size) ;
 /* --- secondary CSparse routines and data structures ----------------------- */
 typedef struct cs_symbolic  /* symbolic Cholesky, LU, or QR analysis */
 {
-    int *pinv ;	    /* inverse row perm. for QR, fill red. perm for Chol */
-    int *q ;	    /* fill-reducing column permutation for LU and QR */
+    int *pinv ;     /* inverse row perm. for QR, fill red. perm for Chol */
+    int *q ;        /* fill-reducing column permutation for LU and QR */
     int *parent ;   /* elimination tree for Cholesky and QR */
-    int *cp ;	    /* column pointers for Cholesky, row counts for QR */
+    int *cp ;       /* column pointers for Cholesky, row counts for QR */
     int *leftmost ; /* leftmost[i] = min(find(A(i,:))), for QR */
-    int m2 ;	    /* # of rows for QR, after adding fictitious rows */
+    int m2 ;        /* # of rows for QR, after adding fictitious rows */
     double lnz ;    /* # entries in L for LU or Cholesky; in V for QR */
     double unz ;    /* # entries in U for LU; in R for QR */
 } css ;
 
 typedef struct cs_numeric   /* numeric Cholesky, LU, or QR factorization */
 {
-    cs *L ;	    /* L for LU and Cholesky, V for QR */
-    cs *U ;	    /* U for LU, R for QR, not used for Cholesky */
-    int *pinv ;	    /* partial pivoting for LU */
-    double *B ;	    /* beta [0..n-1] for QR */
+    cs *L ;         /* L for LU and Cholesky, V for QR */
+    cs *U ;         /* U for LU, R for QR, not used for Cholesky */
+    int *pinv ;     /* partial pivoting for LU */
+    double *B ;     /* beta [0..n-1] for QR */
 } csn ;
 
 typedef struct cs_dmperm_results    /* cs_dmperm or cs_scc output */
 {
-    int *p ;	    /* size m, row permutation */
-    int *q ;	    /* size n, column permutation */
-    int *r ;	    /* size nb+1, block k is rows r[k] to r[k+1]-1 in A(p,q) */
-    int *s ;	    /* size nb+1, block k is cols s[k] to s[k+1]-1 in A(p,q) */
-    int nb ;	    /* # of blocks in fine dmperm decomposition */
+    int *p ;        /* size m, row permutation */
+    int *q ;        /* size n, column permutation */
+    int *r ;        /* size nb+1, block k is rows r[k] to r[k+1]-1 in A(p,q) */
+    int *s ;        /* size nb+1, block k is cols s[k] to s[k+1]-1 in A(p,q) */
+    int nb ;        /* # of blocks in fine dmperm decomposition */
     int rr [5] ;    /* coarse row decomposition */
     int cc [5] ;    /* coarse column decomposition */
 } csd ;
